@@ -291,12 +291,18 @@ inresult_to_lists(pmInResult *result, int **instlist, char ***namelist)
     return n;
 }
 
+/*
+ * Internal variant of pmGetInDom() ... ctxp is not NULL for
+ * internal callers where the current context is already locked, but
+ * NULL for callers from above the PMAPI or internal callers when the
+ * current context is not locked.
+ */
 int
-pmGetInDom(pmInDom indom, int **instlist, char ***namelist)
+pmGetInDom_ctx(__pmContext *ctxp, pmInDom indom, int **instlist, char ***namelist)
 {
     int			sts;
+    int			need_unlock = 0;
     int			i;
-    __pmContext		*ctxp;
     char		*p;
     int			need;
     int			*ilist = NULL;
@@ -314,11 +320,16 @@ pmGetInDom(pmInDom indom, int **instlist, char ***namelist)
 
     if ((sts = pmWhichContext()) >= 0) {
 	int	ctx = sts;
-	ctxp = __pmHandleToPtr(ctx);
 	if (ctxp == NULL) {
-	    sts = PM_ERR_NOCONTEXT;
-	    goto pmapi_return;
+	    ctxp = __pmHandleToPtr(ctx);
+	    if (ctxp == NULL) {
+		sts = PM_ERR_NOCONTEXT;
+		goto pmapi_return;
+	    }
+	    need_unlock = 1;
 	}
+	else
+	    PM_ASSERT_IS_LOCKED(ctxp->c_lock);
 	if (ctxp->c_type == PM_CONTEXT_HOST) {
 	    sts = __pmSendInstanceReq(ctxp->c_pmcd->pc_fd, __pmPtrToHandle(ctxp), indom, PM_IN_NULL, NULL);
 	    if (sts < 0)
@@ -335,7 +346,8 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_CALL);
 		    if ((sts = __pmDecodeInstance(pb, &result)) < 0) {
 			if (pinpdu > 0)
 			    __pmUnpinPDUBuf(pb);
-			PM_UNLOCK(ctxp->c_lock);
+			if (need_unlock)
+			    PM_UNLOCK(ctxp->c_lock);
 			goto pmapi_return;
 		    }
 		    sts = inresult_to_lists(result, instlist, namelist);
@@ -376,12 +388,16 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_CALL);
 		for (i = 0; i < sts; i++)
 		    need += sizeof(char *) + strlen(nametmp[i]) + 1;
 		if ((ilist = (int *)malloc(sts * sizeof(insttmp[0]))) == NULL) {
-		    PM_UNLOCK(ctxp->c_lock);
+		    if (need_unlock)
+			PM_UNLOCK(ctxp->c_lock);
 		    sts = -oserror();
 		    goto pmapi_return;
 		}
 		if ((nlist = (char **)malloc(need)) == NULL) {
-		    PM_UNLOCK(ctxp->c_lock);
+		    free(ilist);
+		    ilist = NULL;
+		    if (need_unlock)
+			PM_UNLOCK(ctxp->c_lock);
 		    sts = -oserror();
 		    goto pmapi_return;
 		}
@@ -396,7 +412,8 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_CALL);
 		}
 	    }
 	}
-	PM_UNLOCK(ctxp->c_lock);
+	if (need_unlock)
+	    PM_UNLOCK(ctxp->c_lock);
     }
 
     if (sts == 0) {
@@ -430,6 +447,14 @@ pmapi_return:
     return sts;
 }
 
+int
+pmGetInDom(pmInDom indom, int **instlist, char ***namelist)
+{
+    int	sts;
+    sts = pmGetInDom_ctx(NULL, indom, instlist, namelist);
+    return sts;
+}
+
 void
 __pmDumpInResult(FILE *f, const pmInResult *irp)
 {
