@@ -168,6 +168,7 @@ extern int __pmIsSecureclientLock(void *) _PCP_HIDDEN;
 extern int __pmIsSecureserverLock(void *) _PCP_HIDDEN;
 extern int __pmIsConnectLock(void *) _PCP_HIDDEN;
 extern int __pmIsExecLock(void *) _PCP_HIDDEN;
+extern int __pmIsThrottleLock(void *) _PCP_HIDDEN;
 #endif
 
 /*
@@ -327,6 +328,7 @@ extern int pmNameAll_ctx(__pmContext *, pmID, char ***) _PCP_HIDDEN;
 extern int pmLookupDesc_ctx(__pmContext *, int, pmID, pmDesc *) _PCP_HIDDEN;
 extern int pmNameInDom_ctx(__pmContext *, pmInDom, int, char **) _PCP_HIDDEN;
 extern int pmLookupInDom_ctx(__pmContext *, pmInDom, const char *) _PCP_HIDDEN;
+extern int pmGetInDom_ctx(__pmContext *, pmInDom, int **, char ***) _PCP_HIDDEN;
 extern int pmGetInDomArchive_ctx(__pmContext *, pmInDom, int **, char ***) _PCP_HIDDEN;
 extern int pmFetch_ctx(__pmContext *, int, pmID *, __pmResult **) _PCP_HIDDEN;
 extern int pmStore_ctx(__pmContext *, const __pmResult *) _PCP_HIDDEN;
@@ -405,7 +407,6 @@ extern __pmTimestamp *__pmLogStartTime(__pmArchCtl *) _PCP_HIDDEN;
 extern int __pmLogSetTime(__pmContext *) _PCP_HIDDEN;
 extern void __pmLogResetInterp(__pmContext *) _PCP_HIDDEN;
 extern void __pmArchCtlFree(__pmArchCtl *) _PCP_HIDDEN;
-extern int __pmLogChangeArchive(__pmContext *, int) _PCP_HIDDEN;
 extern int __pmLogChangeToNextArchive(__pmLogCtl **) _PCP_HIDDEN;
 extern int __pmLogChangeToPreviousArchive(__pmLogCtl **) _PCP_HIDDEN;
 
