@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2023 Red Hat.
+ * Copyright (c) 2016-2024 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -115,6 +115,7 @@ enum {
 	CLUSTER_FCHOST,		/* 91 /sys/class/fc_host metrics */
 	CLUSTER_WWID,		/* 92 multipath aggregated stats */
 	CLUSTER_PRESSURE_IRQ,	/* 93 /proc/pressure/irq metrics */
+	CLUSTER_HUGEPAGES,	/* 94 /sys/kernel/mm/hugepages metrics */
 
 	NUM_CLUSTERS		/* one more than highest numbered cluster */
 };
@@ -141,7 +142,8 @@ enum {
 
 	REFRESH_SYSFS_KERNEL_UEVENTSEQ,
 	REFRESH_SYSFS_KERNEL_EXTFRAG,
-	REFRESH_SYSFS_MODULE_ZSWAPPOOL,
+	REFRESH_SYSFS_MODULE_ZSWAP,
+	REFRESH_SYSFS_KERNEL_VMMEMCTL,
 
 	NUM_REFRESHES		/* one more than highest refresh index */
 };
@@ -193,6 +195,7 @@ enum {
 	INTERRUPT_CPU_INDOM,	/* 40 - per-CPU interrupt lines */
 	SOFTIRQ_CPU_INDOM,	/* 41 - per-CPU soft IRQs */
 	WWID_INDOM,		/* 42 - per-WWID multipath device */
+	HUGEPAGES_INDOM,	/* 43 - hugepages (fixed sizes) */
 
 	NUM_INDOMS		/* one more than highest numbered cluster */
 };
@@ -303,6 +306,9 @@ typedef struct {
     uint64_t		cpu_collision;
     uint64_t		received_rps;
     uint64_t		flow_limit_count;
+    uint64_t		total_backlog;
+    uint64_t		input_qlen;
+    uint64_t		process_qlen;
 } softnet_t;
 
 typedef struct {
