@@ -1,7 +1,7 @@
 /*
  * Linux sysfs_kernel cluster
  *
- * Copyright (c) 2009,2014,2016,2023 Red Hat.
+ * Copyright (c) 2009,2014,2016,2023-2024 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -82,9 +82,19 @@ refresh_sysfs_kernel(sysfs_kernel_t *sk, int *need_refresh)
 	}
     }
 
-    if (need_refresh[REFRESH_SYSFS_MODULE_ZSWAPPOOL]) {
+    if (need_refresh[REFRESH_SYSFS_MODULE_ZSWAP]) {
 	int	fd;
 
+	pmsprintf(buf, sizeof(buf), "%s/%s/zswap/parameters/enabled",
+				    linux_statspath, "sys/module");
+	if ((fd = open(buf, O_RDONLY)) >= 0) {
+	    if ((n = read(fd, buf, sizeof(buf))) > 0) {
+		buf[n-1] = '\0';
+		sscanf(buf, "%c", &sk->zswap_enabled[0]);
+	    }
+	    close(fd);
+	}
+
 	pmsprintf(buf, sizeof(buf), "%s/%s/zswap/parameters/max_pool_percent",
 				    linux_statspath, "sys/module");
 	if ((fd = open(buf, O_RDONLY)) >= 0) {
@@ -96,5 +106,36 @@ refresh_sysfs_kernel(sysfs_kernel_t *sk, int *need_refresh)
 	}
     }
 
+    if (need_refresh[REFRESH_SYSFS_KERNEL_VMMEMCTL]) {
+	unsigned long long value, failed;
+	char name[64];
+	FILE *fp;
+
+	pmsprintf(buf, sizeof(buf), "%s/%s/debug/vmmemctl",
+				    linux_statspath, "sys/kernel");
+	if ((fp = fopen(buf, "r")) != NULL) {
+
+	    while (fgets(buf, sizeof(buf), fp) != NULL) {
+		/*
+		 * Care is needed here: this sysfs file has conflicting
+		 * entries like the following (we want the first case):
+		 * target     :     1023777
+		 * target     :         768 (0 failed)
+		 *
+		 * To tackle this, provide matching on the failed part,
+		 * and discard any line that *does* match there.
+		 */
+		n = sscanf(buf, "%s : %llu (%llu failed)", name, &value, &failed);
+		if (n != 2)
+		    continue;
+		else if (strcmp(name, "current") == 0)
+		    sk->vmmemctl_current = value << _pm_pageshift;
+		else if (strcmp(name, "target") == 0)
+		    sk->vmmemctl_target = value << _pm_pageshift;
+	    }
+	    fclose(fp);
+	}
+    }
+
     return 0;
 }
