@@ -230,27 +230,22 @@ _compress_now()
 #
 if $SHOWME
 then
+    # Exception is for -N where we want to see the output.
+    #
     :
 elif [ "$PROGLOG" = "/dev/tty" ]
 then
-    # special case for debugging ... no salt away previous, no chown, no exec
+    # special case for debugging ... no salt away previous
     #
     :
 else
     # Salt away previous log, if any ...
     #
-    PROGLOGDIR=`dirname "$PROGLOG"`
-    [ -d "$PROGLOGDIR" ] || mkdir_and_chown "$PROGLOGDIR" 775 $PCP_USER:$PCP_GROUP 2>/dev/null
     _save_prev_file "$PROGLOG"
     # After argument checking, everything must be logged to ensure no mail is
     # accidentally sent from cron.  Close stdout and stderr, then open stdout
-    # as our logfile and redirect stderr there too.  Create the log file with
-    # correct ownership first.
+    # as our logfile and redirect stderr there too.
     #
-    # Exception ($SHOWME, above) is for -N where we want to see the output.
-    #
-    touch "$MYPROGLOG"
-    chown $PCP_USER:$PCP_GROUP "$MYPROGLOG" >/dev/null 2>&1
     exec 1>"$MYPROGLOG" 2>&1
 fi
 
@@ -415,13 +410,26 @@ _configure_pmlogger()
 		    elif [ -w "$configfile" ]
 		    then
 			$VERBOSE && echo "Reconfigured: \"$configfile\" (pmlogconf)"
-			chown $PCP_USER:$PCP_GROUP "$tmpconfig" >/dev/null 2>&1
 			eval $MV "$tmpconfig" "$configfile"
 			echo "=== pmlogconf changes @ `date` ==="
 			cat $tmp/diag
 		    else
-			_warning "no write access to pmlogconf file \"$configfile\", skip reconfiguration"
+			# transition problem we're trying to resolve ... configfile may have
+			# been owned by root but in a dir owned by $PCP_USER ... if so we'd
+			# like configfile to end up ownded by $PCP_USER
+			#
+			_warning "no write access to pmlogconf file \"$configfile\""
 			ls -l "$configfile"
+			echo "Trying to remove and replace \"$configfile\" ..."
+			$RM -f "$configfile"
+			if [ -e "$configfile" ]
+			then
+			    echo "Failed, parent directory is ..."
+			    ls -l `dirname "$configfile"`
+			    _warning "skip reconfiguration"
+			else
+			    eval $MV "$tmpconfig" "$configfile"
+			fi
 		    fi
 		else
 		    _warning "pmlogconf failed to reconfigure \"$configfile\""
@@ -450,7 +458,6 @@ _configure_pmlogger()
 	    echo "=== end pmlogconf file ==="
 	else
 	    $VERBOSE && echo "Created: \"$configfile\" (pmlogconf)"
-	    chown $PCP_USER:$PCP_GROUP "$configfile" >/dev/null 2>&1
 	fi
     else
 	$VERBOSE && echo "Botched: \"$configfile\" (pmlogconf)"
@@ -814,7 +821,11 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 	#
 	if [ ! -d "$dir" ]
 	then
-	    mkdir_and_chown "$dir" 775 $PCP_USER:$PCP_GROUP >$tmp/tmp 2>&1
+	    # mode rwxrwxr-x is the default for pcp:pcp dirs
+	    umask 002
+	    mkdir -p -m 0775 "$dir" >$tmp/tmp 2>&1
+	    # reset the default mode to rw-rw-r- for files
+	    umask 022
 	    if [ ! -d "$dir" ]
 	    then
 		cat $tmp/tmp
@@ -825,10 +836,6 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 	    fi
 	fi
 
-	# and the logfile is writeable, if it exists
-	#
-	[ -f "$logfile" ] && chown $PCP_USER:$PCP_GROUP "$logfile" >/dev/null 2>&1
-
 	if cd "$dir"
 	then
 	    :
