@@ -24,6 +24,10 @@
 #
 unset PCP_STDERR
 
+# want mode for mkdir below to reliably be rwxrwxr-x (775)
+#
+umask 022
+
 # constant setup
 #
 tmp=`mktemp -d "$PCP_TMPFILE_DIR/pmlogger_daily.XXXXXXXXX"` || exit 1
@@ -553,27 +557,22 @@ done
 
 if $SHOWME
 then
+    # Exception is for -N where we want to see the output.
+    #
     :
 elif [ "$PROGLOG" = "/dev/tty" ]
 then
-    # special case for debugging ... no salt away previous, no chown, no exec
+    # special case for debugging ... no salt away previous
     #
     :
 else
     # Salt away previous log, if any ...
     #
-    PROGLOGDIR=`dirname "$PROGLOG"`
-    [ -d "$PROGLOGDIR" ] || mkdir_and_chown "$PROGLOGDIR" 775 $PCP_USER:$PCP_GROUP 2>/dev/null
     _save_prev_file "$PROGLOG"
     # After argument checking, everything must be logged to ensure no mail is
     # accidentally sent from cron.  Close stdout and stderr, then open stdout
-    # as our logfile and redirect stderr there too.  Create the log file with
-    # correct ownership first.
+    # as our logfile and redirect stderr there too.
     #
-    # Exception ($SHOWME, above) is for -N where we want to see the output.
-    #
-    touch "$MYPROGLOG"
-    chown $PCP_USER:$PCP_GROUP "$MYPROGLOG" >/dev/null 2>&1
     exec 1>"$MYPROGLOG" 2>&1
 fi
 
@@ -942,7 +941,6 @@ then
 	    echo "*** rotated by $prog: `date`" >>"$NOTICES"
 	    mv -f "$NOTICES" "$NOTICES.old"
 	    echo "Started by $prog: `date`" >"$NOTICES"
-	    chown $PCP_USER:$PCP_GROUP "$NOTICES" >/dev/null 2>&1
 	fi
     fi
 fi
@@ -1350,7 +1348,11 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 	#
 	if [ ! -d "$dir" ]
 	then
-	    mkdir_and_chown "$dir" 775 $PCP_USER:$PCP_GROUP >$tmp/tmp 2>&1
+	    # mode rwxrwxr-x is the default for pcp:pcp dirs
+	    umask 002
+	    mkdir -p -m 0775 "$dir" >$tmp/tmp 2>&1
+	    # reset the default mode to rw-rw-r- for files
+	    umask 022
 	    if [ ! -d "$dir" ]
 	    then
 		cat $tmp/tmp
@@ -2097,7 +2099,11 @@ p
 			       -e s/LOCALHOSTNAME/$dirhostname/`"
 		if [ ! -d "$auto_dir" -a "$auto_dir" != "$last_mkdir" ]
 		then
-		    mkdir_and_chown "$auto_dir" 775 $PCP_USER:$PCP_GROUP >$tmp/tmp 2>&1
+		    # mode rwxrwxr-x is the default for pcp:pcp dirs
+		    umask 002
+		    mkdir -p -m 0775 "$auto_dir" >$tmp/tmp 2>&1
+		    # reset the default mode to rw-rw-r- for files
+		    umask 022
 		    if [ ! -d "$auto_dir" ]
 		    then
 			cat $tmp/tmp
